/*
	 stage00.c 

	 Copyright (C) 1997-1999, NINTENDO Co,Ltd.
*/

#include <assert.h>
#include <nusys.h>
#include "graphic.h"
#include "main.h"

// #include "EuroVilla_Hilltop_72.h"
#include "checkerboard_gamma.h"
#include "checkerboard.h"

void shadetri(Dynamic* dynamicp);
void MyDrawN(Dynamic* dynamicp);
void MyDrawFloor(Dynamic *dynamicp);
void MyDrawNSide();

float GLOBAL_ROT = 0.0f;
float GLOBAL_ROT_INC = 0.1f;
unsigned char GLOBAL_DITHER_FILTER_STATE = 1;
unsigned char GLOBAL_GAMMA_ENABLE = 1;
u32 GLOBAL_DITHER_MODE = G_CD_BAYER;

u8 framecounter = 0;
u8 GLOBAL_DITHER_FLICKER = 0;
static Mtx floor_mat;

void updateGame00(void) {
	/* Increment Animation Timer */
	GLOBAL_ROT += GLOBAL_ROT_INC;

	nuContDataGetEx(contdata, 0);

	if (contdata[0].button & A_BUTTON) {
		GLOBAL_ROT_INC += 0.1f;
		if (GLOBAL_ROT_INC > 10.0f) GLOBAL_ROT_INC = 10.0f;
	}
	if (contdata[0].button & B_BUTTON) {
		GLOBAL_ROT_INC -= 0.1f;
		if (GLOBAL_ROT_INC < 0.0f) GLOBAL_ROT_INC = 0.0f;
	}

	if (contdata[0].trigger & R_TRIG) {
		GLOBAL_DITHER_FILTER_STATE = !GLOBAL_DITHER_FILTER_STATE;
	}

	if (contdata[0].trigger & U_CBUTTONS) {
		GLOBAL_DITHER_MODE = G_CD_DISABLE;
	} else if (contdata[0].trigger & L_CBUTTONS) {
		GLOBAL_DITHER_MODE = G_CD_BAYER;
	} else if (contdata[0].trigger & R_CBUTTONS) {
		GLOBAL_DITHER_MODE = G_CD_MAGICSQ;
	} else if (contdata[0].trigger & D_CBUTTONS) {
		GLOBAL_DITHER_MODE = G_CD_NOISE;
	}

	if (contdata[0].trigger & Z_TRIG) {
		GLOBAL_DITHER_FLICKER = (GLOBAL_DITHER_FLICKER + 1) & 0x1;
	}

	if (contdata[0].trigger & START_BUTTON) {
		GLOBAL_GAMMA_ENABLE = (GLOBAL_GAMMA_ENABLE + 1) & 0x1;
	}
}

void guRotateCat(Mtx *inMat, float angle, float x, float y, float z, Mtx *outMat) {
	Mtx tempMat;
	guRotate(&tempMat, angle, x, y, z);
	guMtxCatL(&tempMat, inMat, outMat);
}

void guTranslateCat(Mtx *inMat, float x, float y, float z, Mtx *outMat) {
	Mtx tempMat;
	guTranslate(&tempMat, x, y, z);
	guMtxCatL(&tempMat, inMat, outMat);
}

void guScaleCat(Mtx *inMat, float x, float y, float z, Mtx *outMat) {
	Mtx tempMat;
	guScale(&tempMat, x, y, z);
	guMtxCatL(&tempMat, inMat, outMat);
}

inline void guIdentity(Mtx *inMat) {
	guMtxIdent(inMat);
}

/* Make the display list and activate the task. */

void makeDL00(void)
{
	Mtx tempMat;
	Mtx rot2;
	u16 precision = 1;
	char conbuf[20]; 
	char *s_on = " on";
	char *s_off = "off";
	char *s_dither_labels[4] = {
		"Bayer   ",
		"Magic SQ",
		"Noise   ",
		"Disabled"
	};
	int label_index = 0;

	u32 dither_mode = GLOBAL_DITHER_MODE;
	framecounter++;

	if (GLOBAL_DITHER_FLICKER) {
		if (GLOBAL_DITHER_MODE == G_CD_BAYER && (framecounter&0x1)) {
			dither_mode = G_CD_MAGICSQ;
		} else if (GLOBAL_DITHER_MODE == G_CD_MAGICSQ && (framecounter&0x1)) {
			dither_mode = G_CD_BAYER;
		}
	}
	

	/* Turn on Gamma correction */
	if (GLOBAL_GAMMA_ENABLE) {
		osViSetSpecialFeatures(OS_VI_GAMMA_ON);
	} else {
		osViSetSpecialFeatures(OS_VI_GAMMA_OFF);
	}
	if (GLOBAL_DITHER_FILTER_STATE) {
		osViSetSpecialFeatures(OS_VI_DITHER_FILTER_ON);
	} else {
		osViSetSpecialFeatures(OS_VI_DITHER_FILTER_OFF);
	}


	/* Specify the display list buffer  */
	glistp = gfx_glist;

	/*  The initialization of RCP  */
	gfxRCPInit();

	/* Clear the frame buffer and the Z-buffer  */
	gfxClearCfb();

	/* The set of projection modeling matrices  */
	/*
	guOrtho(&gfx_dynamic.projection,
		-(float)SCREEN_WD/2.0F, (float)SCREEN_WD/2.0F,
		-(float)SCREEN_HT/2.0F, (float)SCREEN_HT/2.0F,
		1.0F, 10.0F, 1.0F);
	*/
	guPerspective(&gfx_dynamic.projection,
	&precision,
	45.0f, 320.0f/240.0f,
	1.0f, 100.0f, 1.0f);
	guLookAt(&tempMat, 0, 12, 24, 0, 0, 0, 0, 1, 0);
	guMtxCatL(&tempMat, &gfx_dynamic.projection, &gfx_dynamic.projection);

	guRotate(&gfx_dynamic.modeling, GLOBAL_ROT, 0.0f, 1.0f, 0.0F); // ANGLE, X, Y, Z
	guRotate(&rot2, GLOBAL_ROT*0.31, 1.0f, 0.0f, 0.0f);
	guMtxCatL(&gfx_dynamic.modeling, &rot2, &gfx_dynamic.modeling);

	guIdentity(&floor_mat);
	guScaleCat(&floor_mat, 50.0f/1000.0f, 1.0, 50.0f/1000.0f, &floor_mat);
	guTranslateCat(&floor_mat, 0.0f, -7.0f, 0.0f, &floor_mat);
	guRotateCat(&floor_mat, GLOBAL_ROT, 0.0f, 1.0f, 0.0f, &floor_mat);


	/* Draw a square  */
	// shadetri(&gfx_dynamic);
	// gDPSetColorDither(glistp++, G_CD_BAYER);
	gDPSetColorDither(glistp++, dither_mode);
	MyDrawN(&gfx_dynamic);
	MyDrawFloor(&gfx_dynamic);
	// shadetri(&gfx_dynamic);

	/* End the construction of the display list  */
	gDPFullSync(glistp++);
	gSPEndDisplayList(glistp++);

	/* Check if all are put in the array  */
	assert(glistp - gfx_glist < GFX_GLIST_LEN);

	/* Activate the RSP task.  Switch display buffers at the end of the task. */
	nuGfxTaskStart(gfx_glist,
		 (s32)(glistp - gfx_glist) * sizeof (Gfx),
		 NU_GFX_UCODE_F3DEX , NU_SC_NOSWAPBUFFER);

	nuDebTaskPerfBar0(1,200,NU_SC_NOSWAPBUFFER);

  if(contpattern & 0x1)
    {
		/* Change character representation positions */
		nuDebConTextPos(0,2,23);
		sprintf(conbuf,"GLOBAL_ROT_INC=%2.1f",GLOBAL_ROT_INC);
		nuDebConCPuts(0, conbuf);

		switch (dither_mode) {
			case G_CD_BAYER:
				label_index = 0;
				break;
			case G_CD_MAGICSQ:
				label_index = 1; 
				break;
			case G_CD_NOISE:
				label_index = 2;
				break;
			case G_CD_DISABLE: 
				label_index = 3;
				break;
			default:
				break;
		}

		nuDebConTextPos(0,2,24);
		sprintf(conbuf,"Filter: %s Dither: %s",GLOBAL_DITHER_FILTER_STATE?s_on:s_off, s_dither_labels[label_index]);
		nuDebConCPuts(0, conbuf);

		nuDebConTextPos(0,2,1);
		sprintf(conbuf, "%08x - %08x Code Segment",(u32)_codeSegmentStart, (u32)_codeSegmentEnd);
		nuDebConCPuts(0, conbuf);

		// I'm counting on the compiler pushing these in the order they're declared
		nuDebConTextPos(0,2,2);
		sprintf(conbuf, "%08x - %08x local stack vars", (u32)(&tempMat), (u32)(&label_index));
		nuDebConCPuts(0, conbuf);

		nuDebConTextPos(0,2,3);
		sprintf(conbuf, "%08x Framebuffer Start", (u32)(nuGfxCfb[0]));
		nuDebConCPuts(0, conbuf);
    }
  else
    {
      nuDebConTextPos(0,2,24);
      nuDebConCPuts(0, "Controller1 not connect");
    }
    
  /* Display characters on the frame buffer */
  nuDebConDisp(NU_SC_SWAPBUFFER);

}

/* The vertex coordinate  */
static Vtx shade_vtx[] =  {
	{        -1,  1, 1,   0,  0, 0,   0, 127, 0,    0xFf	}, //  0: normal for top
	{         1,  1, 1,   0,  0, 0,   127, 0, 0,    0xFf	}, //  1: normal for right
	{         1, -1, 1,   0,  0, 0,   0, 0, 127,    0xFf	}, //  2: normal for front
	{        -1, -1, 1,   0,  0, 0,   0,-128, 0,    0xFf	}, //  3: normal for bottom
	{        -1,  1, -1,   0,  0, 0, -128, 0, 0,    0xFf	}, //  4: normal for left
	{         1,  1, -1,   0,  0, 0,  0, 0,-128,    0xFf	}, //  5: normal for back
	{         1, -1, -1,   0,  0, 0,  0, 128, 0,    0xFf	}, //  6: unused normal
	{        -1, -1, -1,   0,  0, 0,  0, 128, 0,    0xFf	}, //  7: unused normal
};
/* this needs to be global !!! */
Lights1 my_light = gdSPDefLights1(
	57, 65, 81,
	219, 217, 196, 20, 100, 20
);

/* Draw a square  */
void shadetri(Dynamic* dynamicp)
{

	gSPMatrix(glistp++,OS_K0_TO_PHYSICAL(&(dynamicp->projection)),
		G_MTX_PROJECTION|G_MTX_LOAD|G_MTX_NOPUSH);
	gSPMatrix(glistp++,OS_K0_TO_PHYSICAL(&(dynamicp->modeling)),
		G_MTX_MODELVIEW|G_MTX_LOAD|G_MTX_NOPUSH);

	gDPPipeSync(glistp++);
	gDPSetCycleType(glistp++,G_CYC_1CYCLE);
	gDPSetRenderMode(glistp++,G_RM_AA_ZB_OPA_SURF, G_RM_AA_ZB_OPA_SURF2);
	gSPClearGeometryMode(glistp++,0xFFFFFFFF);
	// gSPSetGeometryMode(glistp++,G_SHADE| G_SHADING_SMOOTH);
	gSPSetLights1(glistp++, my_light);
	gSPSetGeometryMode(glistp++, G_LIGHTING|G_ZBUFFER|G_SHADE|G_CULL_BACK);


	gSPVertex(glistp++,&(shade_vtx[0]),8, 0);
	// Pay extra careful attention to which vertex you're using to store
	// the flat shaded polygon's normal vector.

	// front
	gSP2Triangles(glistp++, 1, 0, 2, 2, 2, 0, 3, 0);
	// back
	gSP2Triangles(glistp++, 4, 5, 7, 1, 7, 5, 6, 1);
	// left
	gSP2Triangles(glistp++, 0, 4, 3, 1, 3, 4, 7, 1);
	// right
	gSP2Triangles(glistp++, 6, 1, 2, 1, 5, 1, 6, 1);
	// bottom
	gSP2Triangles(glistp++, 6, 3, 7, 1, 2, 3, 6, 1);
	// top
	gSP2Triangles(glistp++, 4, 0, 5, 1, 5, 0, 1, 1);
}


Lights1 greenMat, blueMat, redMat, yellowMat;

void LightToMaterial1(Lights1 *inLight, Lights1 *outLight, unsigned char r, unsigned char g, unsigned char b) {

	if (GLOBAL_GAMMA_ENABLE) {
		r = (r*r)>>8;
		g = (g*g)>>8;
		b = (b*b)>>8;
	}

	outLight->a.l.col[0] = outLight->a.l.colc[0] = (inLight->a.l.col[0]*r)>>8;
	outLight->a.l.col[1] = outLight->a.l.colc[1] = (inLight->a.l.col[1]*g)>>8;
	outLight->a.l.col[2] = outLight->a.l.colc[2] = (inLight->a.l.col[2]*b)>>8;
	outLight->l[0].l.col[0] = outLight->l[0].l.colc[0] = (inLight->l[0].l.col[0]*r)>>8;
	outLight->l[0].l.col[1] = outLight->l[0].l.colc[1] = (inLight->l[0].l.col[1]*g)>>8;
	outLight->l[0].l.col[2] = outLight->l[0].l.colc[2] = (inLight->l[0].l.col[2]*b)>>8;
	outLight->l[0].l.dir[0] = inLight->l[0].l.dir[1];
	outLight->l[0].l.dir[1] = inLight->l[0].l.dir[1];
	outLight->l[0].l.dir[2] = inLight->l[0].l.dir[2];
}

unsigned char nLogoInitialized = 0;


/* vertex coords for N side */
static Vtx n_front_vtx[] =  {
	{        -4, -4,  4,   0,  0, 0,   0, 0, 127,    0xFf	},
	{        -2, -4,  4,   0,  0, 0,   0, 0, 127,    0xFf	},
	{        -2,  1,  4,   0,  0, 0,   0, 0, 127,    0xFf	},
	{        -4,  4,  4,   0,  0, 0,   0, 0, 127,    0xFf	},
	{        -2,  4,  4,   0,  0, 0,   0, 0, 127,    0xFf	},
	{         2, -4,  4,   0,  0, 0,   0, 0, 127,    0xFf	},
	{         4, -4,  4,   0,  0, 0,   0, 0, 127,    0xFf	},
	{         2, -1,  4,   0,  0, 0,   0, 0, 127,    0xFf	},
	{         2,  4,  4,   0,  0, 0,   0, 0, 127,    0xFf	},
	{         4,  4,  4,   0,  0, 0,   0, 0, 127,    0xFf	},

	{        -4, -1,  2,   0,  0, 0,   0, 0, -127,    0xFf	},
	{        -2, -1,  2,   0,  0, 0,   0, 0, -127,    0xFf	},
	{        -2,  1,  2,   0,  0, 0,   0, 0, -127,    0xFf	},
	{        -4,  4,  2,   0,  0, 0,   0, 0, -127,    0xFf	},
	{        -2,  4,  2,   0,  0, 0,   0, 0, -127,    0xFf	},
	{         2, -4,  2,   0,  0, 0,   0, 0, -127,    0xFf	},
	{         4, -4,  2,   0,  0, 0,   0, 0, -127,    0xFf	},
	{         2, -1,  2,   0,  0, 0,   0, 0, -127,    0xFf	},
	{         2,  1,  2,   0,  0, 0,   0, 0, -127,    0xFf	},
	{         4,  1,  2,   0,  0, 0,   0, 0, -127,    0xFf	},
};
static Vtx n_back_vtx[20];
static Vtx n_right_vtx[20];
static Vtx n_left_vtx[20];

static Vtx n_top_bottom_vtx[] = {
	{        -4,  4,  4,   0,  0, 0,   0, 127, 0,    0xFf	},
	{        -2,  4,  4,   0,  32<<5, 0,   0, 127, 0,    0xFf	},
	{        -4,  4,  2,   0,  0, 32<<5,   0, 127, 0,    0xFf	},
	{        -2,  4,  2,   0,  32<<5, 32<<5,   0, 127, 0,    0xFf	},

	{        -4,  4, -2,   0,  0, 0,   0, 127, 0,    0xFf	},
	{        -2,  4, -2,   0,  32<<5, 0,   0, 127, 0,    0xFf	},
	{        -4,  4, -4,   0,  0, 32<<5,   0, 127, 0,    0xFf	},
	{        -2,  4, -4,   0,  32<<5, 32<<5,   0, 127, 0,    0xFf	},

	{         2,  4,  4,   0,  0, 0,   0, 127, 0,    0xFf	},
	{         4,  4,  4,   0,  32<<5, 0,   0, 127, 0,    0xFf	},
	{         2,  4,  2,   0,  0, 32<<5,   0, 127, 0,    0xFf	},
	{         4,  4,  2,   0,  32<<5, 32<<5,   0, 127, 0,    0xFf	},

	{         2,  4, -2,   0,  0, 0,   0, 127, 0,    0xFf	},
	{         4,  4, -2,   0,  32<<5, 0,   0, 127, 0,    0xFf	},
	{         2,  4, -4,   0,  0, 32<<5,   0, 127, 0,    0xFf	},
	{         4,  4, -4,   0,  32<<5, 32<<5,   0, 127, 0,    0xFf	},

	// this data will be overwritten

	{        -4,  4,  4,   0,  0, 0,   0, 127, 0,    0xFf	},
	{        -2,  4,  4,   0,  0, 0,   0, 127, 0,    0xFf	},
	{        -4,  4,  2,   0,  0, 0,   0, 127, 0,    0xFf	},
	{        -2,  4,  2,   0,  0, 0,   0, 127, 0,    0xFf	},

	{        -4,  4, -2,   0,  0, 0,   0, 127, 0,    0xFf	},
	{        -2,  4, -2,   0,  0, 0,   0, 127, 0,    0xFf	},
	{        -4,  4, -4,   0,  0, 0,   0, 127, 0,    0xFf	},
	{        -2,  4, -4,   0,  0, 0,   0, 127, 0,    0xFf	},

	{         2,  4,  4,   0,  0, 0,   0, 127, 0,    0xFf	},
	{         4,  4,  4,   0,  0, 0,   0, 127, 0,    0xFf	},
	{         2,  4,  2,   0,  0, 0,   0, 127, 0,    0xFf	},
	{         4,  4,  2,   0,  0, 0,   0, 127, 0,    0xFf	},

	{         2,  4, -2,   0,  0, 0,   0, 127, 0,    0xFf	},
	{         4,  4, -2,   0,  0, 0,   0, 127, 0,    0xFf	},
	{         2,  4, -4,   0,  0, 0,   0, 127, 0,    0xFf	},
	{         4,  4, -4,   0,  0, 0,   0, 127, 0,    0xFf	},
};

static Vtx n_slopes_vtx[] = {
	{        -2,  4,  4,   0,  0, 0,   100, 80, 0,    0xFf	},
	{         2, -1,  4,   0,  0, 0,   100, 80, 0,    0xFf	},
	{        -2,  4,  2,   0,  0, 0,   100, 80, 0,    0xFf	},
	{         2, -1,  2,   0,  0, 0,   100, 80, 0,    0xFf	},
	{        -2,  1,  4,   0,  0, 0,   -100, -80, 0,    0xFf	},
	{         2, -4,  4,   0,  0, 0,   -100, -80, 0,    0xFf	},
	{        -2,  1,  2,   0,  0, 0,   -100, -80, 0,    0xFf	},
	{         2, -4,  2,   0,  0, 0,   -100, -80, 0,    0xFf	},

	// this data will be overwritten

	{        -2,  4,  4,   0,  0, 0,   100, 80, 0,    0xFf	},
	{         2, -1,  4,   0,  0, 0,   100, 80, 0,    0xFf	},
	{        -2,  4,  2,   0,  0, 0,   100, 80, 0,    0xFf	},
	{         2, -1,  2,   0,  0, 0,   100, 80, 0,    0xFf	},
	{        -2, -1,  4,   0,  0, 0,   -100, -80, 0,    0xFf	},
	{         2, -4,  4,   0,  0, 0,   -100, -80, 0,    0xFf	},
	{         2, -1,  2,   0,  0, 0,   -100, -80, 0,    0xFf	},
	{         2, -4,  2,   0,  0, 0,   -100, -80, 0,    0xFf	},
	{        -2,  4,  4,   0,  0, 0,   100, 80, 0,    0xFf	},
	{         2, -1,  4,   0,  0, 0,   100, 80, 0,    0xFf	},
	{        -2,  4,  2,   0,  0, 0,   100, 80, 0,    0xFf	},
	{         2, -1,  2,   0,  0, 0,   100, 80, 0,    0xFf	},
	{        -2, -1,  4,   0,  0, 0,   -100, -80, 0,    0xFf	},
	{         2, -4,  4,   0,  0, 0,   -100, -80, 0,    0xFf	},
	{         2, -1,  2,   0,  0, 0,   -100, -80, 0,    0xFf	},
	{         2, -4,  2,   0,  0, 0,   -100, -80, 0,    0xFf	},
	{        -2,  4,  4,   0,  0, 0,   100, 80, 0,    0xFf	},
	{         2, -1,  4,   0,  0, 0,   100, 80, 0,    0xFf	},
	{        -2,  4,  2,   0,  0, 0,   100, 80, 0,    0xFf	},
	{         2, -1,  2,   0,  0, 0,   100, 80, 0,    0xFf	},
	{        -2, -1,  4,   0,  0, 0,   -100, -80, 0,    0xFf	},
	{         2, -4,  4,   0,  0, 0,   -100, -80, 0,    0xFf	},
	{         2, -1,  2,   0,  0, 0,   -100, -80, 0,    0xFf	},
	{         2, -4,  2,   0,  0, 0,   -100, -80, 0,    0xFf	},

};

static Vtx floor_verts[32];

void InitFloorVerts() {
	float radius = 1000.0f;
	float pi = 3.1415926535897932384626433832795f;
	int i;

	for (i=0;i<31;i++) {
		floor_verts[i].v.ob[0] = (s16)(radius * cosf(i*2.0f*pi/32.0f));
		floor_verts[i].v.ob[1] = 0;
		floor_verts[i].v.ob[2] = (s16)(-radius * sinf(i*2.0f*pi/32.0f));
		floor_verts[i].v.flag = 0;
		floor_verts[i].v.tc[0] = (s16)((5.0f*(32<<6)) * cosf(i*2.0f*pi/32.0f));
		floor_verts[i].v.tc[1] = (s16)(-(5.0f*(32<<6)) * sinf(i*2.0f*pi/32.0f));
		floor_verts[i].v.cn[0] = 0x00;
		floor_verts[i].v.cn[1] = 0x00;
		floor_verts[i].v.cn[2] = 0x00;
		floor_verts[i].v.cn[3] = 0xff;
	}
	floor_verts[31].v.ob[0] = floor_verts[31].v.ob[1] = floor_verts[31].v.ob[2] = 0;
	floor_verts[31].v.flag = 0;
	floor_verts[31].v.tc[0] = floor_verts[31].v.tc[1] = 0;
	floor_verts[31].v.cn[0] = 0xff;
	floor_verts[31].v.cn[1] = 0xff;
	floor_verts[31].v.cn[2] = 0xff;
	floor_verts[31].v.cn[3] = 0xff;
}


void InitOtherSides() {
	int i;
	for (i=0;i<20;i++) {
		// back
		n_back_vtx[i].n.ob[0] = -n_front_vtx[i].n.ob[0];
		n_back_vtx[i].n.ob[1] =  n_front_vtx[i].n.ob[1];
		n_back_vtx[i].n.ob[2] = -n_front_vtx[i].n.ob[2];
		n_back_vtx[i].n.flag =  n_front_vtx[i].n.flag;
		n_back_vtx[i].n.tc[0] = n_front_vtx[i].n.tc[0];
		n_back_vtx[i].n.tc[1] = n_front_vtx[i].n.tc[1];
		n_back_vtx[i].n.n[0] = 0;
		n_back_vtx[i].n.n[1] = 0;
		n_back_vtx[i].n.n[2] = -n_front_vtx[i].n.n[2];
		n_back_vtx[i].n.a = 0xff;

		// right
		n_right_vtx[i].n.ob[0] =  n_front_vtx[i].n.ob[2];
		n_right_vtx[i].n.ob[1] =  n_front_vtx[i].n.ob[1];
		n_right_vtx[i].n.ob[2] = -n_front_vtx[i].n.ob[0];
		n_right_vtx[i].n.flag = n_front_vtx[i].n.flag;
		n_right_vtx[i].n.tc[0] = n_front_vtx[i].n.tc[0];
		n_right_vtx[i].n.tc[1] = n_front_vtx[i].n.tc[1];
		n_right_vtx[i].n.n[0] = n_front_vtx[i].n.n[2];
		n_right_vtx[i].n.n[1] = n_front_vtx[i].n.n[1];
		n_right_vtx[i].n.n[2] = -n_front_vtx[i].n.n[0];
		n_right_vtx[i].n.a = 0xff;

		// left
		n_left_vtx[i].n.ob[0] =  n_back_vtx[i].n.ob[2];
		n_left_vtx[i].n.ob[1] =  n_back_vtx[i].n.ob[1];
		n_left_vtx[i].n.ob[2] = -n_back_vtx[i].n.ob[0];
		n_left_vtx[i].n.flag = n_back_vtx[i].n.flag;
		n_left_vtx[i].n.tc[0] = n_back_vtx[i].n.tc[0];
		n_left_vtx[i].n.tc[1] = n_back_vtx[i].n.tc[1];
		n_left_vtx[i].n.n[0] = n_back_vtx[i].n.n[2];
		n_left_vtx[i].n.n[1] = n_back_vtx[i].n.n[1];
		n_left_vtx[i].n.n[2] = -n_back_vtx[i].n.n[0];
		n_left_vtx[i].n.a = 0xff;

	}

	for (i=0;i<16;i++) {
		// bottom
		n_top_bottom_vtx[i+16].n.ob[0] = -n_top_bottom_vtx[i].n.ob[0];
		n_top_bottom_vtx[i+16].n.ob[1] = -n_top_bottom_vtx[i].n.ob[1];
		n_top_bottom_vtx[i+16].n.ob[2] = n_top_bottom_vtx[i].n.ob[2];
		n_top_bottom_vtx[i+16].n.n[1] = -127;
		n_top_bottom_vtx[i+16].n.tc[0] = n_top_bottom_vtx[i].n.tc[0];
		n_top_bottom_vtx[i+16].n.tc[1] = n_top_bottom_vtx[i].n.tc[1];
	}

	for (i=0;i<8;i++) {
		// slope rot 90
		n_slopes_vtx[i+8].n.ob[0] = n_slopes_vtx[i].n.ob[2];
		n_slopes_vtx[i+8].n.ob[1] = n_slopes_vtx[i].n.ob[1];
		n_slopes_vtx[i+8].n.ob[2] = -n_slopes_vtx[i].n.ob[0];
		n_slopes_vtx[i+8].n.n[0] = n_slopes_vtx[i].n.n[2];
		n_slopes_vtx[i+8].n.n[1] = n_slopes_vtx[i].n.n[1];
		n_slopes_vtx[i+8].n.n[2] = -n_slopes_vtx[i].n.n[0];
		// slope rot 180
		n_slopes_vtx[i+16].n.ob[0] = -n_slopes_vtx[i].n.ob[0];
		n_slopes_vtx[i+16].n.ob[1] = n_slopes_vtx[i].n.ob[1];
		n_slopes_vtx[i+16].n.ob[2] = -n_slopes_vtx[i].n.ob[2];
		n_slopes_vtx[i+16].n.n[0] = -n_slopes_vtx[i].n.n[0];
		n_slopes_vtx[i+16].n.n[1] = n_slopes_vtx[i].n.n[1];
		n_slopes_vtx[i+16].n.n[2] = -n_slopes_vtx[i].n.n[2];
		// slopes rot -90
		n_slopes_vtx[i+24].n.ob[0] = -n_slopes_vtx[i].n.ob[2];
		n_slopes_vtx[i+24].n.ob[1] = n_slopes_vtx[i].n.ob[1];
		n_slopes_vtx[i+24].n.ob[2] = n_slopes_vtx[i].n.ob[0];
		n_slopes_vtx[i+24].n.n[0] = -n_slopes_vtx[i].n.n[2];
		n_slopes_vtx[i+24].n.n[1] = n_slopes_vtx[i].n.n[1];
		n_slopes_vtx[i+24].n.n[2] = n_slopes_vtx[i].n.n[0];
	}
}

void MyDrawFloor(Dynamic *dynamicp) {
	int i;

	if (!nLogoInitialized) {
		InitOtherSides();
		InitFloorVerts();
		nLogoInitialized = 1;
	}
	
	gSPMatrix(glistp++, OS_K0_TO_PHYSICAL(&(floor_mat)),
		G_MTX_MODELVIEW|G_MTX_LOAD|G_MTX_NOPUSH);
	
	gDPPipeSync(glistp++);
	gDPSetCycleType(glistp++, G_CYC_1CYCLE);
	gDPSetRenderMode(glistp++,G_RM_AA_ZB_OPA_SURF, G_RM_AA_ZB_OPA_SURF2);
	gSPClearGeometryMode(glistp++,0xFFFFFFFF);
	gSPSetGeometryMode(glistp++, G_ZBUFFER|G_SHADE|G_SHADING_SMOOTH|G_CULL_BACK);

    gDPSetCombineMode (glistp++,G_CC_MODULATERGB, G_CC_MODULATERGB);
    gDPSetTexturePersp (glistp++,G_TP_PERSP);
    gDPSetTextureLOD (glistp++,G_TL_TILE);
    gDPSetTextureFilter (glistp++,G_TF_BILERP);
    gDPSetTextureConvert(glistp++,G_TC_FILT);
    gDPSetTextureLUT (glistp++,G_TT_NONE);
    gSPTexture(glistp++,0x8000, 0x8000, 0, G_TX_RENDERTILE, G_ON);

	gSPVertex(glistp++, floor_verts, 32, 0);

    /* Read the texture  */
	if (GLOBAL_GAMMA_ENABLE) {
		gDPLoadTextureBlock(glistp++,
			checkerboard_gamma,
			G_IM_FMT_RGBA,
			G_IM_SIZ_16b,
			32, 32, 0,
			G_TX_WRAP | G_TX_NOMIRROR, G_TX_WRAP | G_TX_NOMIRROR,
			5, 5, G_TX_NOLOD, G_TX_NOLOD);
	} else {
		gDPLoadTextureBlock(glistp++,
			checkerboard,
			G_IM_FMT_RGBA,
			G_IM_SIZ_16b,
			32, 32, 0,
			G_TX_WRAP | G_TX_NOMIRROR, G_TX_WRAP | G_TX_NOMIRROR,
			5, 5, G_TX_NOLOD, G_TX_NOLOD);
	}
	
	for (i=0; i<30; i+=2) {
		gSP2Triangles(glistp++, i, i+1, 31, 0, i+1, i+2, 31, 0);
	}
	gSP1Triangle(glistp++, 30, 0, 31, 0);



	gDPSetCombineMode(glistp++, G_CC_SHADE, G_CC_SHADE);
}


void MyDrawN(Dynamic* dynamicp) {

	if (!nLogoInitialized) {
		InitOtherSides();
		InitFloorVerts();
		nLogoInitialized = 1;
	}

	// Set up all colors
	LightToMaterial1(&my_light, &greenMat, 78, 245, 78);
	LightToMaterial1(&my_light, &blueMat, 78, 78, 245);
	LightToMaterial1(&my_light, &redMat, 245, 78, 78);
	LightToMaterial1(&my_light, &yellowMat, 245, 245, 78);

	gSPMatrix(glistp++,OS_K0_TO_PHYSICAL(&(dynamicp->projection)),
		G_MTX_PROJECTION|G_MTX_LOAD|G_MTX_NOPUSH);
	gSPMatrix(glistp++,OS_K0_TO_PHYSICAL(&(dynamicp->modeling)),
		G_MTX_MODELVIEW|G_MTX_LOAD|G_MTX_NOPUSH);

	gDPPipeSync(glistp++);
	gDPSetCycleType(glistp++,G_CYC_1CYCLE);
	gDPSetRenderMode(glistp++,G_RM_AA_ZB_OPA_SURF, G_RM_AA_ZB_OPA_SURF2);
	gSPClearGeometryMode(glistp++,0xFFFFFFFF);
	// gSPSetGeometryMode(glistp++,G_SHADE| G_SHADING_SMOOTH);
	gSPSetGeometryMode(glistp++, G_LIGHTING|G_ZBUFFER|G_SHADE|G_SHADING_SMOOTH|G_CULL_BACK);

	gSPSetLights1(glistp++, greenMat);

	gSPVertex(glistp++, &(n_front_vtx[0]), 20, 0);

	gSP2Triangles(glistp++, 0, 1, 2, 0, 0, 2, 3, 0);
	gSP2Triangles(glistp++, 3, 2, 4, 0, 4, 2, 7, 0);
	gSP2Triangles(glistp++, 7, 2, 5, 0, 7, 5, 6, 0);
	gSP2Triangles(glistp++, 7, 6, 9, 0, 7, 9, 8, 0);

	gSP2Triangles(glistp++, 12, 11, 10, 0, 12, 10, 13, 0);
	gSP2Triangles(glistp++, 14, 12, 13, 0, 17, 12, 14, 0);
	gSP2Triangles(glistp++, 15, 12, 17, 0, 16, 15, 17, 0);
	gSP2Triangles(glistp++, 19, 16, 17, 0, 18, 19, 17, 0);


	gSPVertex(glistp++, &(n_back_vtx[0]), 20, 0);

	gSP2Triangles(glistp++, 0, 1, 2, 0, 0, 2, 3, 0);
	gSP2Triangles(glistp++, 3, 2, 4, 0, 4, 2, 7, 0);
	gSP2Triangles(glistp++, 7, 2, 5, 0, 7, 5, 6, 0);
	gSP2Triangles(glistp++, 7, 6, 9, 0, 7, 9, 8, 0);

	gSP2Triangles(glistp++, 12, 11, 10, 0, 12, 10, 13, 0);
	gSP2Triangles(glistp++, 14, 12, 13, 0, 17, 12, 14, 0);
	gSP2Triangles(glistp++, 15, 12, 17, 0, 16, 15, 17, 0);
	gSP2Triangles(glistp++, 19, 16, 17, 0, 18, 19, 17, 0);

	gSPSetLights1(glistp++, blueMat);

	gSPVertex(glistp++, &(n_right_vtx[0]), 20, 0);

	gSP2Triangles(glistp++, 0, 1, 2, 0, 0, 2, 3, 0);
	gSP2Triangles(glistp++, 3, 2, 4, 0, 4, 2, 7, 0);
	gSP2Triangles(glistp++, 7, 2, 5, 0, 7, 5, 6, 0);
	gSP2Triangles(glistp++, 7, 6, 9, 0, 7, 9, 8, 0);

	gSP2Triangles(glistp++, 12, 11, 10, 0, 12, 10, 13, 0);
	gSP2Triangles(glistp++, 14, 12, 13, 0, 17, 12, 14, 0);
	gSP2Triangles(glistp++, 15, 12, 17, 0, 16, 15, 17, 0);
	gSP2Triangles(glistp++, 19, 16, 17, 0, 18, 19, 17, 0);

	gSPVertex(glistp++, &(n_left_vtx[0]), 20, 0);

	gSP2Triangles(glistp++, 0, 1, 2, 0, 0, 2, 3, 0);
	gSP2Triangles(glistp++, 3, 2, 4, 0, 4, 2, 7, 0);
	gSP2Triangles(glistp++, 7, 2, 5, 0, 7, 5, 6, 0);
	gSP2Triangles(glistp++, 7, 6, 9, 0, 7, 9, 8, 0);

	gSP2Triangles(glistp++, 12, 11, 10, 0, 12, 10, 13, 0);
	gSP2Triangles(glistp++, 14, 12, 13, 0, 17, 12, 14, 0);
	gSP2Triangles(glistp++, 15, 12, 17, 0, 16, 15, 17, 0);
	gSP2Triangles(glistp++, 19, 16, 17, 0, 18, 19, 17, 0);

	gSPSetLights1(glistp++, yellowMat);

	gSPVertex(glistp++, &(n_top_bottom_vtx[0]), 32, 0);

	gSP2Triangles(glistp++, 0, 1, 2, 0, 2, 1, 3, 0);
	gSP2Triangles(glistp++, 4, 5, 6, 0, 6, 5, 7, 0);
	gSP2Triangles(glistp++, 8, 9, 10, 0, 10, 9, 11, 0);
	gSP2Triangles(glistp++, 12, 13, 14, 0, 14, 13, 15, 0);

	gSP2Triangles(glistp++, 16, 17, 18, 0, 18, 17, 19, 0);
	gSP2Triangles(glistp++, 20, 21, 22, 0, 22, 21, 23, 0);
	gSP2Triangles(glistp++, 24, 25, 26, 0, 26, 25, 27, 0);
	gSP2Triangles(glistp++, 28, 29, 30, 0, 30, 29, 31, 0);

	gDPPipeSync(glistp++);

	gSPSetLights1(glistp++, redMat);

	gSPVertex(glistp++, &(n_slopes_vtx[0]), 32, 0);

	gSP2Triangles(glistp++, 0, 1, 2, 0, 2, 1, 3, 0);
	gSP2Triangles(glistp++, 4, 6, 5, 0, 5, 6, 7, 0);

	gSP2Triangles(glistp++, 8, 9, 10, 0, 10, 9, 11, 0);
	gSP2Triangles(glistp++, 12, 14, 13, 0, 13, 14, 15, 0);

	gSP2Triangles(glistp++, 16, 17, 18, 0, 18, 17, 19, 0);
	gSP2Triangles(glistp++, 20, 22, 21, 0, 21, 22, 23, 0);

	gSP2Triangles(glistp++, 24, 25, 26, 0, 26, 25, 27, 0);
	gSP2Triangles(glistp++, 28, 30, 29, 0, 29, 30, 31, 0);


}


void MyDrawNSide() {
}

